@@ -98,14 +98,7 @@ class Field(object):
             self.units = unitconverters_map[self.fieldtype]
         else:
             raise ValueError("Unsupported mesh type. Choose either: 'spherical' or 'flat'")
-        if timestamps is not None:
-            # Check whether flattened or not
-            if all(isinstance(f, np.ndarray) for f in timestamps):
-                self.timestamps = np.array([stamp for f in timestamps for stamp in f])
-            if all(isinstance(stamp, np.datetime64) for stamp in timestamps):
-                self.timestamps = timestamps
-        else:
-            self.timestamps = timestamps
+        self.timestamps = timestamps
         if type(interp_method) is dict:
             if self.name in interp_method:
                 self.interp_method = interp_method[self.name]
@@ -313,8 +306,7 @@ class Field(object):
                 for findex in range(len(data_filenames)):
                     for f in [data_filenames[findex]] * len(timestamps[findex]):
                         dataFiles.append(f)
-                timestamps = np.array([stamp for file in timestamps for stamp in file])
-                timeslices = timestamps
+                timeslices = np.array([stamp for file in timestamps for stamp in file])
                 time = timeslices
             elif netcdf_engine == 'xarray':
                 with NetcdfFileBuffer(data_filenames, dimensions, indices, netcdf_engine) as filebuffer:
@@ -1083,7 +1075,11 @@ class Field(object):
 
     def computeTimeChunk(self, data, tindex):
         g = self.grid
-        timestamp = None if self.timestamps is None else self.timestamps[tindex]
+        timestamp = self.timestamps
+        if timestamp is not None:
+            summedlen = np.cumsum([len(ls) for ls in self.timestamps])
+            timestamp = self.timestamps[np.where(g.ti + tindex < summedlen)[0][0]]
+
         filebuffer = NetcdfFileBuffer(self.dataFiles[g.ti + tindex], self.dimensions, self.indices,
                                       self.netcdf_engine, timestamp=timestamp,
                                       interp_method=self.interp_method,
