@@ -232,8 +232,12 @@ class ParticleFile(object):
             if pset.size == 0:
                 logger.warning("ParticleSet is empty on writing as array at time %g" % time)
             else:
-                tol = 1e-8
-                pset_towrite = [p for p in pset if p.dt * p.time < p.dt * time + tol and np.isfinite(p.id)]
+                if deleted_only:
+                    pset_towrite = pset
+                elif pset[0].dt > 0:
+                    pset_towrite = [p for p in pset if time <= p.time < time + p.dt and np.isfinite(p.id)]
+                else:
+                    pset_towrite = [p for p in pset if time + p.dt < p.time <= time and np.isfinite(p.id)]
                 if len(pset_towrite) > 0:
                     for var in self.var_names:
                         data_dict[var] = np.array([getattr(p, var) for p in pset_towrite])
